unit clsMySQL;{&Use32+}
(****************************************************************************)
(*									    *)
(*  MYSQL CLASS LIBRARY FOR OS/2 AND WIN32 - CROSS PLATFORM PASCAL CLASSES  *)
(*									    *)
(*	    COPYRIGHT 1998 ANTONY T CURTIS   ALL RIGHTS RESERVED	    *)
(*	    antony.curtis@olcs.net	     http://nuts.ml.org/	    *)
(*									    *)
(*  USAGE LICENSE : AUTHOR MUST BE PROPERLY ACKNOWLEDGED IN PRODUCTS WHICH  *)
(*   MAKE USE OF THIS CODE. NO WARRANITES EITHER EXPRESSED OR IMPLIED ARE   *)
(*     APPLICABLE AS TO THE FITNESS OF THIS CODE - IT IS SUPPLIED AS IS     *)
(*									    *)
(*	    SUPPORT IS SUPPLIED AT THE DISCRETION OF THE AUTHOR 	    *)
(*									    *)
(****************************************************************************)
interface
uses SysUtils, libMySQL;

type
  MySQLData		= Class;
  MySQL 		= Class
  private
    session		: libMySQL.MYSQL;
    last_error		: PChar;
    uptimestr		: Array[0..40] of Char;
    function	getServerInfo:PChar;
    function	getClientInfo:PChar;
    function	getProtoInfo:Word;
    function	getStackInfo:PChar;
    function	getConnectInfo:PChar;
    function	getServerUptime:PChar;
  public
    constructor Create(const host,user,password:PChar);
    constructor Create2(const host,user,password:PChar;port:Word;
			const unix_socket:PChar;client_flag:Word);
    destructor	Destroy; override;

    Function	Shutdown:Boolean;
    Function	Refresh(options:Word):Boolean;
    Function	KillTID(tid:Cardinal):Boolean;
    Function	ServerStatus:PChar;

    Function	UseDB(const db:PChar):Boolean;
    Function	CreateDB(const db:PChar):Boolean;
    Function	DropDB(const db:PChar):Boolean;
    Function	AskDB(const query:PChar;args:Array of Const):Boolean;
    Function	Result:MySQLData;

    Function	ListDBs(const wild:PChar):MySQLData;
    Function	ListTables(const wild:PChar):MySQLData;
    Function	ListFields(const table,wild:PChar):MySQLData;
    Function	ListThreads:MySQLData;

    property	InsertID:Cardinal read session.insert_id;
    property	Affected:Cardinal read session.affected_rows;
    property	ThreadID:Word read session.thread_id;

    property	ErrorNum:Word read session.net.last_errno;
    property	ErrorStr:PChar read last_error;
    property	InfoStr:PChar read session.info;

    property	ServerVersion:PChar read getServerInfo;
    property	ClientVersion:PChar read getClientInfo;
    property	ProtocolVersion:Word read getProtoInfo;
    property	StackVersion:PChar read getStackInfo;
    property	ConnectionInfo:PChar read getConnectInfo;
    property	ServerUptime:PChar read getServerUptime;

    property	unix_socket:PChar read session.unix_socket;
    property	tcp_socket:Word read session.port;

    // For handling BLOBs...
    class function Escape_String(sto:PChar;const from:PChar;from_length:Word):Word;
    class function Unescape_String(sto:PChar;const from:PChar):Word;
  end;

  MySQLFields		= Class;
  MySQLData		= Class
  private
    sqlfields		: MySQLFields;
    sqldata		: libMySQL.PMYSQL_RES;
    current		: libMySQL.PMYSQL_ROW;
    currentidx		: Word;
    Function	getFields:MySQLFields;
    procedure	setField(const fname:PChar;data:PChar);
    function	getField(const fname:PChar):PChar;
    function	FieldLen(const fname:PChar):Word;
    procedure	setField2(idx:Word;data:PChar);
    function	getField2(idx:Word):PChar;
    function	FieldLen2(idx:Word):Word;
    function	getEOF:Boolean;
  public
    Constructor Create(data:libMySQL.PMYSQL_RES);
    Destructor	Destroy; override;

    Function	SeekTo(row:Word):Boolean;
    Function	NextRow:Boolean;
    Function	LastRow:Integer;

    Function	DumpData:PChar; // Free with StrDispose

    property	EOF:Boolean read getEOF;

    property	Fields:MySQLFields read getFields;
    property	Data[const fname:PChar]:PChar read getField write setField; default;
    property	DataIdx[idx:Word]:PChar read getField2 write setField2;
    property	Length[const fname:PChar]:Word read FieldLen;
    property	LenIdx[idx:Word]:Word read FieldLen2;
  end;

  MySQLFields		= Class
  private
    sqldata		: MySQLData;
    current		: libMySQL.PMYSQL_FIELD;
    currentidx		: Word;
    maxidx		: Integer;
    function	getField(idx:Word):PChar;
    function	FieldLen(idx:Word):Word;
    function	FieldWidth(idx:Word):Word;
  public
    Constructor Create(dataset:MySQLData);
    Destructor	Destroy; override;
    function	FieldIdx(const fname:PChar):Integer;
    property	Field[idx:Word]:PChar read getField; default;
    property	Length[idx:Word]:Word read FieldLen;
    property	Width[idx:Word]:Word read FieldWidth;
    property	LastIdx:Integer read maxidx;
  end;

  EMySQL			= Class(Exception);
  EOutOfMemory			= Class(EMySQL);
  EConnection			= Class(EMySQL);
  ESocketCreate 		= Class(EConnection);
  ELocalConnect 		= Class(EConnection);
  EHostConnect			= Class(EConnection);
  EServerGone			= Class(EConnection);
  EServerLost			= Class(EConnection);
  EBadHostInfo			= Class(EConnection);
  ELocalhost			= Class(EConnection);
  ERemotehost			= Class(EConnection);
  EUnknownhost			= Class(EConnection);
  EProtocol			= Class(EMySQL);
  EHandshake			= Class(EProtocol);
  EOutOfSync			= Class(EProtocol);

implementation
uses NetLib;

type
  TException		= Class of Exception;
const
  MySQLExcept		: Array[CR_UNKNOWN_ERROR..CR_COMMANDS_OUT_OF_SYNC] of TException=(
    EMySQL,ESocketCreate,ELocalConnect,EHostConnect,ESocketCreate,
    EUnknownHost,EServerGone,EProtocol,EOutOfMemory,
    EBadHostInfo,ELocalhost,ERemotehost,EHandshake,
    EServerLost,EOutOfSync
  );

procedure RaiseError(error:Word;info:PChar); inline;
begin
  if (error>=Low(MySQLExcept)) and (error<=High(MySQLExcept)) then
    raise MySQLExcept[error].CreateFmt('MySQL:%d:%s',[error,info])
  else
    raise MySQLExcept[error].CreateFmt('MySQL:%d:Unknown Error:%s',[error,info])
end;

function Check(sql:MySQL;res:Integer):Boolean; inline;
begin
  Result := res = 0;
  if res < 0 then begin
    if (sql.ErrorNum>=Low(MySQLExcept)) and (sql.ErrorNum<=High(MySQLExcept)) then
      raise MySQLExcept[sql.ErrorNum].CreateFmt('MySQL:%d:%s',[sql.ErrorNum,sql.ErrorStr])
    else
      raise MySQLExcept[sql.ErrorNum].CreateFmt('MySQL:%d:Unknown Error:%s',[sql.ErrorNum,sql.ErrorStr])
  end;
end;

class function MySQL.Escape_String;
begin
  Result := mysql_escape_string(sto,from,from_length);
end;

class function MySQL.Unescape_String;
begin
  Result := mysql_unescape_string(sto,from);
end;

constructor MySQL.Create;
begin
  inherited Create;
  last_error := @session.net.last_error;
  if mysql_connect(@session,host,user,password)=nil then fail;
end;

constructor MySQL.Create2;
begin
  inherited Create;
  last_error := @session.net.last_error;
  if mysql_real_connect(@session,host,user,password,port,unix_socket,client_flag)=nil then fail;
end;

destructor MySQL.Destroy;
begin
  mysql_close(@session);
  inherited Destroy;
end;

function MySQL.Shutdown;
begin
  Result := Check(self,mysql_shutdown(@session));
  Free;
end;

function MySQL.Refresh;
begin
  Result := check(self,mysql_refresh(@session,options));
end;

function MySQL.KillTID(tid:Cardinal):Boolean;
begin
  Result := check(self,mysql_kill(@session,tid));
end;

function MySQL.ServerStatus:PChar;
begin
  Result := mysql_stat(@session);
end;

function MySQL.UseDB;
begin
  Result := check(self,mysql_select_db(@session,db));
end;

function MySQL.CreateDB;
begin
  Result := check(self,mysql_create_db(@session,db));
end;

function MySQL.DropDB;
begin
  Result := check(self,mysql_drop_db(@session,db));
end;

function MySQL.AskDB(const query:PChar;args:Array of Const):Boolean;
var
  buff			: PChar;
  buffsize		: Word;
  i			: Integer;
begin
  if query=nil then begin
    Result := False;
    exit;
  end;
  buffsize := StrLen(query);
  for i := Low(args) to High(args) do with args[i] do begin
    case vtype of
      vtInteger        : inc(buffsize,11);
      vtBoolean        : inc(buffsize,5);
      vtChar	       : inc(buffsize,1);
      vtExtended       : inc(buffsize,20);
      vtString	       : if vString<>nil then inc(buffsize,length(vString^));
      vtPointer        : inc(buffsize,8);
      vtPChar	       : if vPChar<>nil then inc(buffsize,StrLen(vPChar));
    end;
  end;
  buffsize := (buffsize + 8191) and not 8191;
  buff := AllocMem(buffsize);
  i := mysql_query(@session,StrLFmt(buff,buffsize,query,args));
  FreeMem(buff,buffsize);
  Result := check(self,i);
end;

Function MySQL.Result;
begin
  Result := MySQLData.Create(mysql_store_result(@session));
  if Result=nil then raiseerror(ErrorNum,ErrorStr);
end;

Function MySQL.ListDBs;
begin
  Result := MySQLData.Create(mysql_list_dbs(@session,wild));
  if Result=nil then raiseerror(ErrorNum,ErrorStr);
end;

Function MySQL.ListTables;
begin
  Result := MySQLData.Create(mysql_list_tables(@session,wild));
  if Result=nil then raiseerror(ErrorNum,ErrorStr);
end;

Function MySQL.ListFields;
begin
  Result := MySQLData.Create(mysql_list_fields(@session,table,wild));
  if Result=nil then raiseerror(ErrorNum,ErrorStr);
end;

Function MySQL.ListThreads;
begin
  Result := MySQLData.Create(mysql_list_processes(@session));
  if Result=nil then raiseerror(ErrorNum,ErrorStr);
end;

function MySQL.getServerInfo;
begin
  Result := mysql_get_server_info(@session);
end;

function MySQL.getClientInfo;
begin
  Result := mysql_get_client_info;
end;

function MySQL.getProtoInfo;
begin
  Result := mysql_get_proto_info(@session);
end;

function MySQL.getStackInfo;
begin
  Result := getinetversion;
  if Result=nil then Result := 'unknown stack';
end;

function MySQL.getConnectInfo;
begin
  Result := mysql_get_host_info(@session);
end;

function MySQL.getServerUptime;
var
  stat		: PChar;
begin
  //stat := ServerStatus;
  Result := uptimestr;
end;

Constructor MySQLFields.Create;
begin
  inherited Create;
  if dataset=nil then fail;
  if dataset.sqlfields<>nil then fail;
  dataset.sqlfields := self;
  sqldata := dataset;
  maxidx := mysql_num_fields(sqldata.sqldata)-1;
  currentidx := $FFFF;
end;

Destructor MySQLFields.Destroy;
begin
  if (sqldata<>nil)and(sqldata.sqlfields=self) then sqldata.sqlfields := nil;
  inherited Destroy;
end;

function MySQLFields.getField;
begin
  if idx>LastIdx then begin
    Result := nil;
    exit;
  end;
  if (current=nil)or(currentidx<>idx) then begin
    mysql_field_seek(sqldata.sqldata,idx);
    current := mysql_fetch_field(sqldata.sqldata);
    currentidx := idx;
  end;
  if current=nil then Result := '' else Result := current.fname;
end;

function MySQLFields.FieldLen;
begin
  if idx>LastIdx then begin
    Result := 0;
    exit;
  end;
  if (current=nil)or(currentidx<>idx) then begin
    mysql_field_seek(sqldata.sqldata,idx);
    current := mysql_fetch_field(sqldata.sqldata);
    currentidx := idx;
  end;
  if current=nil then Result := 0 else Result := current.max_length;
end;

function MySQLFields.FieldWidth;
begin
  if idx>LastIdx then begin
    Result := 0;
    exit;
  end;
  if (current=nil)or(currentidx<>idx) then begin
    mysql_field_seek(sqldata.sqldata,idx);
    current := mysql_fetch_field(sqldata.sqldata);
    currentidx := idx;
  end;
  if current=nil then Result := 0 else Result := current.length;
end;

function MySQLFields.FieldIdx;
var
  myidx 		: Word;
  field 		: libMySQL.PMYSQL_FIELD;
begin
  if (current<>nil)and(StrIComp(current.fname,fname)=0) then begin
    Result := currentidx;
    exit;
  end;
  mysql_field_seek(sqldata.sqldata,0); myidx := 0;
  field := mysql_fetch_field(sqldata.sqldata);
  while field<>nil do begin
    if StrIComp(fname,field.fname)=0 then begin
      Result := myidx;
      current := field;
      currentidx := myidx;
      exit;
    end;
    inc(myidx);
    field := mysql_fetch_field(sqldata.sqldata);
  end;
  Result := -1;
end;

Constructor MySQLData.Create;
var
  field 		: PMYSQL_FIELD;
begin
  inherited Create;
  if data=nil then fail;
  sqldata := data;
  sqlfields := nil;
  current := nil;
  SeekTo(0);
end;

Destructor MySQLData.Destroy;
begin
  if sqlfields<>nil then sqlfields.Destroy;
  mysql_free_result(sqldata);
  inherited Destroy;
end;

Function MySQLData.SeekTo;
begin
  if (current=nil)or(currentidx<>row) then begin
    mysql_data_seek(sqldata,row);
    current := mysql_fetch_row(sqldata);
    currentidx := row;
  end;
  Result := current<>nil;
end;

Function MySQLData.LastRow;
begin
  Result := mysql_num_rows(sqldata) -1;
end;

function MySQLData.getEOF;
begin
  Result := current<>nil;
end;

Function MySQLData.NextRow;
begin
  if current<>nil then begin
    current := mysql_fetch_row(sqldata);
    inc(currentidx);
  end;
  Result := current<>nil;
end;

function MySQLData.getFields;
begin
  if sqlfields<>nil then
    Result := sqlfields
  else
    Result := MySQLFields.Create(self);
end;

function MySQLData.getField2;
begin
  if (idx > Fields.LastIdx)or(current = nil) then begin
    Result := nil;
    exit;
  end;
  Result := current^[idx];
end;

procedure MySQLData.setField2;
begin
  if (idx > Fields.LastIdx)or(current = nil) then exit;
  if data=nil then data := '';
  StrLCopy(current^[idx],data,Fields.Width[idx]);
end;

function MySQLData.FieldLen2;
begin
  if (idx > Fields.LastIdx)or(current = nil) then begin
    Result := 0;
    exit;
  end;
  Result := Fields.Width[idx];
end;

procedure MySQLData.setField;
var
  idx			: Integer;
begin
  idx := Fields.FieldIdx(fname);
  if idx>=0 then setField2(idx,data);
end;

function MySQLData.getField;
var
  idx			: Integer;
begin
  idx := Fields.FieldIdx(fname);
  if idx>=0 then
    Result := getField2(idx)
  else
    Result := nil;
end;

function MySQLData.FieldLen;
var
  idx			: Integer;
begin
  idx := Fields.FieldIdx(fname);
  if idx>=0 then
    Result := FieldLen2(idx)
  else
    Result := 0;
end;


Function Max(a,b:Integer):Integer;
begin
  if a>b then Result := a else Result := b;
end;

Function MySQLData.DumpData; // Ugly - but it works.
var
  buff,bp,bi		: PChar;
  buffsize		: Word;
  i,j,rows,k		: Integer;
  lnsize		: Word;
  field 		: PMYSQL_FIELD;
  widths,wi		: PWORD;
  row			: PMYSQL_ROW;
begin
  j := Fields.LastIdx;
  Result := nil; lnsize := 3;
  if j<0 then exit;
  widths := AllocMem((j+1)*SizeOf(Word)); wi := widths;
  for i := 0 to j do begin
    wi^ := Max(Fields.Length[i],StrLen(Fields[i]));
    inc(lnsize,3 + wi^); inc(wi);
  end;
  rows := LastRow; buffsize := 1 + 4 * lnsize;
  if rows>=0 then inc(buffsize,(rows+1)*lnsize);
  buffsize := (buffsize + 15) and not 15;
  buff := AllocMem(buffsize);
  if buff = nil then exit; bp := buff; bi := buff;

  FillChar(bp^,lnsize,'-'); inc(bp,lnsize-2); bp^ := #13; inc(bp); bp^ := #10; inc(bp);

  mysql_field_seek(sqldata,0);
  FillChar(bp^,lnsize,' ');
  bp^ := '|'; inc(bp); bi^ := '+'; inc(bi); wi := widths;
  for i := 0 to j do begin
    StrECopy(bp+1,Fields[i])^ := ' ';
    inc(bp,2+wi^); inc(bi,2+wi^); inc(wi);
    bp^ := '|'; inc(bp); bi^ := '+'; inc(bi);
  end;
  bp^ := #13; inc(bp); bp^ := #10; inc(bp);

  Move(buff^,bp^,lnsize); inc(bp,lnsize);

  mysql_data_seek(sqldata,0);
  row := mysql_fetch_row(sqldata);
  while row <> nil do begin
    bp^ := '|'; inc(bp); wi := widths;
    for i := 0 to j do begin
      fillChar(bp^,wi^+2,32); inc(bp);
      if row^[i]<>nil then StrECopy(bp,row^[i])^:=' ';
      inc(bp,wi^+1); bp^ := '|'; inc(bp); inc(wi);
    end;
    bp^ := #13; inc(bp); bp^ := #10; inc(bp);
    row := mysql_fetch_row(sqldata);
  end;

  Move(buff^,bp^,lnsize); inc(bp,lnsize);
  bp^ := #0;

  Result := StrNew(buff);
  FreeMem(buff,buffsize)
end;

end.
