program mysqladmin;
(* Copyright Abandoned 1996 TCX DataKonsult AB & Monty Program KB & Detron HB
   This file is public domain and comes with NO WARRANTY of any kind *)
uses Use32, libMySQL, sysMySQL, Strings;
(* maintaince of mysql databases *)

const
  ADMIN_VERSION 	= '6.9';

const
  interval		: Integer	= 0;
  option_quick		: Boolean	= False;
  interrupted		: Boolean	= False;
  new_line		: Boolean	= False;
  option_silent 	: Boolean	= False;


procedure print_version; forward;
procedure usage; forward;
function  execute_commands(mysql:PMYSQL;args:array of const):Boolean;
//static sig_handler endprog(int signal_number);
function  create_db(mysql:PMYSQL;db:PChar):Integer;
function  drop_db(mysql:PMYSQL;db:PChar):Integer;
procedure nice_time(sec:Cardinal;buff:PChar);
procedure print_header(result:PMYSQL_RES);
procedure print_top(result:PMYSQL_RES);
procedure print_row(result:PMYSQL_RES;cur:MYSQL_ROW);

(* The order of commands must be the same as command_names, except ADMIN_ERROR*)
type
  commands		= (
		ADMIN_ERROR, ADMIN_CREATE, ADMIN_DROP, ADMIN_SHUTDOWN,
		ADMIN_RELOAD, ADMIN_REFRESH, ADMIN_VER, ADMIN_PROCESSLIST,
		ADMIN_STATUS, ADMIN_KILL, ADMIN_DEBUG, ADMIN_VARIABLES,
		ADMIN_FLUSH_LOGS, ADMIN_FLUSH_HOSTS, ADMIN_FLUSH_TABLES,
		ADMIN_PASSWORD
  );
const
  command_names 	: Array[commands] of PChar = ( '',
			   'create','drop','shutdown','reload','refresh',
			   'version', 'processlist','status','kill','debug',
			   'variables','flush-logs','flush-hosts',
			   'flush-tables','password');
//TYPELIB commands={array_elements(command_names)-1,"commands",command_names};

(*  long_options	  : Array[] of Options = (
    ('debug'
static struct option long_options[] =
{
  {"debug",	optional_argument, 0, '#'},
  {"force",	no_argument,	   0, 'f'},
  {"help",	no_argument,	   0, '?'},
  {"host",	required_argument, 0, 'h'},
  {"password",	optional_argument, 0, 'p'},
  {"port",	required_argument, 0, 'P'},
  {"silent",	no_argument,	   0, 's'},
  {"socket",	required_argument, 0, 'S'},
  {"sleep",	required_argument, 0, 'i'},
#ifndef DONT_ALLOW_USER_CHANGE
  {"user",	required_argument, 0, 'u'},
#endif
  {"version",	no_argument,	   0, 'V'},
  {0, 0, 0, 0}
};
  *)

int main(int argc,char *argv[])
{
  int	c, error = 0,option_index=0;
  MYSQL mysql;
  char	*host = NULL,*password=0,*user=0;
  bool tty_password=0;
  MY_INIT(argv[0]);

  while ((c=getopt_long(argc,argv,"h:i:p::u:#::P:sS:fq?V",long_options,
			&option_index)) != EOF)
  {
    switch(c) {
    case 'h':
      host = optarg;
      break;
    case 'q':                                   (* Allow old 'q' option *)
    case 'f':
      option_quick++;
      break;
    case 'p':
      if (optarg)
      {
	password=my_strdup(optarg,MYF(MY_FAE));
	while (*optarg) *optarg++= 'x';         (* Destroy argument *)
      }
      else
	tty_password=1;
      break;
#ifndef DONT_ALLOW_USER_CHANGE
    case 'u':
      user=optarg;
      break;
#endif
    case 'i':
      interval=atoi(optarg);
      break;
    case 'P':
      mysql_port= (unsigned int) atoi(optarg);
      break;
    case 's':
      option_silent=1;
      break;
    case 'S':
      mysql_unix_port= optarg;
      break;
    case '#':
      DBUG_PUSH(optarg ? optarg : "d:t:o,/tmp/mysqladmin.trace");
      break;
    case 'V':
      print_version();
      exit(0);
      break;
    default:
      fprintf(stderr,"Illegal option character '%c'\n",opterr);
      (* Fall throught *)
    case '?':
    case 'I':                                   (* Info *)
      error++;
      break;
    }
  }
  argc-=optind;
  argv+=optind;
  if (error || argc == 0)
  {
    usage();
    exit(1);
  }
  if (tty_password)
    password=get_tty_password(NullS);

  VOID(signal(SIGINT,endprog)); 		(* Here if abort *)
  VOID(signal(SIGTERM,endprog));		(* Here if abort *)

  if (!mysql_connect(&mysql,host,user,password))
  {
    error=1;
    if (!option_silent)
    {
      if (!host)
	host=LOCAL_HOST;
      my_printf_error(0,"connect to server at '%s' failed\nerror: '%s'",
		      MYF(ME_BELL), host, mysql_error(&mysql));
      if (mysql_errno(&mysql) == CR_CONNECTION_ERROR)
      {
	fprintf(stderr,"Check that mysqld is running and that the socket: '%s' exists!\n",mysql_unix_port);
      }
      else if (mysql_errno(&mysql) == CR_CONN_HOST_ERROR ||
	       mysql_errno(&mysql) == CR_UNKNOWN_HOST)
      {
	fprintf(stderr,"Check that mysqld is running on %s and that the port is %d.\nYou can check this by doing 'telnet %s %d'\n",
		host, mysql_port, host, mysql_port);
      }
    }
  }
  else
  {
    error=0;
    while (!interrupted)
    {
      new_line=0;
      if (execute_commands(&mysql,argc,argv) && !option_quick)
      {
	error=1;
	break;
      }
      if (interval)
      {
	sleep(interval);
	if (new_line)
	  puts("");
      }
      else
	break;
    }
    mysql_close(&mysql);
  }
  my_free(password,MYF(MY_ALLOW_ZERO_PTR));
  my_end(0);
  exit(error);
  return 0;
}

static sig_handler endprog(int signal_number __attribute__((unused)))
{
  interrupted=1;
}


function execute_commands(mysql:PMYSQL;int argc, char **argv):Boolean;
{
  char *stat;

  for (; argc > 0 ; argv++,argc--)
  {
    switch (find_type(argv[0],&commands,2)) {
    case ADMIN_CREATE:
      if (argc < 2)
      {
	my_printf_error(0,"Too few arguments to create",MYF(ME_BELL));
	return 1;
      }
      else if (create_db(mysql,argv[1]))
	return 1;
      else
      {
	argc--; argv++;
      }
      break;
    case ADMIN_DROP:
      if (argc < 2)
      {
	my_printf_error(0,"Too few arguments to drop",MYF(ME_BELL));
	return 1;
      }
      else if (drop_db(mysql,argv[1]) > 0)
	return 1;
      else
      {
	argc--; argv++;
      }
      break;
    case ADMIN_SHUTDOWN:
      if (mysql_shutdown(mysql))
      {
	my_printf_error(0,"shutdown failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    case ADMIN_RELOAD:
      if (mysql_refresh(mysql,REFRESH_GRANT) < 0)
      {
	my_printf_error(0,"reload failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    case ADMIN_REFRESH:
      if (mysql_refresh(mysql,(uint) ~REFRESH_GRANT) < 0)
      {
	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    case ADMIN_VER:
      new_line=1;
      print_version();
      puts("TCX Datakonsult AB, by Monty\n");
      printf("Server version\t\t%s\n", mysql_get_server_info(mysql));
      printf("Protocol version\t%d\n", mysql_get_proto_info(mysql));
      printf("Connection\t\t%s\n",mysql_get_host_info(mysql));
      if (mysql->unix_socket)
	printf("UNIX socket\t\t%s\n", mysql->unix_socket);
      else
	printf("TCP port\t\t%d\n", mysql->port);
      stat=mysql_stat(mysql);
      {
	char *pos,buff[40];
	ulong sec;
	pos=strchr(stat,' ');
	*pos++=0;
	printf("%s\t\t\t",stat);			(* print label *)
	if ((stat=str2int(pos,10,0,LONG_MAX,(long*) &sec)))
	{
	  nice_time(sec,buff);
	  puts(buff);				(* print nice time *)
	  while (*stat == ' ') stat++;          (* to next info *)
	}
      }
      putc('\n',stdout);
      if (stat)
	puts(stat);
      break;
    case ADMIN_PROCESSLIST:
    {
      MYSQL_RES *result;
      MYSQL_ROW row;

      if (!(result=mysql_list_processes(mysql)))
      {
	my_printf_error(0,"process list failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      print_header(result);
      while ((row=mysql_fetch_row(result)))
	print_row(result,row);
      print_top(result);
      mysql_free_result(result);
      new_line=1;
      break;
    }
    case ADMIN_STATUS:
      stat=mysql_stat(mysql);
      if (stat)
	puts(stat);
      break;
    case ADMIN_KILL:
      {
	uint error=0;
	char *pos;
	if (argc < 2)
	{
	  my_printf_error(0,"Too few arguments to 'kill'",MYF(ME_BELL));
	  return 1;
	}
	pos=argv[1];
	for (;;)
	{
	  if (mysql_kill(mysql,(ulong) atol(pos)))
	  {
	    my_printf_error(0,"kill failed on %d; error: '%s'",MYF(ME_BELL),
			    atol(pos), mysql_error(mysql));
	    error=1;
	  }
	  if (!(pos=strchr(pos,',')))
	    break;
	  pos++;
	}
	argc--; argv++;
	if (error)
	  return error;
	break;
      }
    case ADMIN_DEBUG:
      if (mysql_dump_debug_info(mysql))
      {
	my_printf_error(0,"debug failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    case ADMIN_VARIABLES:
    {
      MYSQL_RES *res;
      MYSQL_ROW row;

      new_line=1;
      if (mysql_query(mysql,"show variables") ||
	  !(res=mysql_store_result(mysql)))
      {
	my_printf_error(0,"unable to show variables; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      print_header(res);
      while ((row=mysql_fetch_row(res)))
	print_row(res,row);
      print_top(res);
      mysql_free_result(res);
      break;
    }
    case ADMIN_FLUSH_LOGS:
    {
      if (mysql_refresh(mysql,REFRESH_LOG))
      {
	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    }
    case ADMIN_FLUSH_HOSTS:
    {
      if (mysql_refresh(mysql,REFRESH_HOSTS))
      {
	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    }
    case ADMIN_FLUSH_TABLES:
    {
      if (mysql_refresh(mysql,REFRESH_TABLES))
      {
	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
			mysql_error(mysql));
	return 1;
      }
      break;
    }
    case ADMIN_PASSWORD:
    {
      char buff[256],crypted_pw[256];

      if(argc < 2)
      {
	my_printf_error(0,"Too few arguments to change password",MYF(ME_BELL));
	return 1;
      }
      if(sizeof(buff)-60 < strlen(argv[1])) (* Crypted passwd *)
      {
	fprintf(stderr,"Too long password: %s",argv[1]);
	return 1;
      }
      make_scrambled_password(crypted_pw,argv[1]);
      sprintf(buff,"set OPTION password='%s'",crypted_pw);

      if(mysql_query(mysql,"set OPTION sql_log_off=1"))
      {
	my_printf_error(0,
			"Can't set logoff while changing password; error: '%s'",
			MYF(ME_BELL),mysql_error(mysql));
	return 1;
      }
      if (mysql_query(mysql,"set OPTION password"))
      {
	my_printf_error(0,"unable to change password; error: '%s'",
			MYF(ME_BELL),mysql_error(mysql));
	return 1;
      }
      if (mysql_query(mysql,"set OPTION sql_log_off=0"))
      {
	my_printf_error(0,
			"Can't set logon after changed password; error: '%s'",
			MYF(ME_BELL),mysql_error(mysql));
	return 1;
      }
      argc--; argv++;
      break;
    }
    default:
      my_printf_error(0,"Unknown command: '%s'",MYF(ME_BELL),argv[0]);
      return 1;
    }
  }
  return 0;
}

procedure print_version;
begin
  Write(Format('%s  Ver %s Distrib %s, for %s on %s'#13#10,[my_progname,ADMIN_VERSION,
	 MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE]));
end;

procedure usage;
begin
  print_version;
  writeln('TCX Datakonsult AB, by Monty');
  writeln('This software comes with NO WARRANTY: see the file PUBLIC for details.'#13#10);
  writeln('Administer program for the mysqld demon (PASCAL VERSION)');
  write(Format('Usage: %s [OPTIONS] command command....'#13#10,[my_progname]));
  write(PChar(#13#10+
	'  -#, --debug=...       Output debug log. Often this is ''d:t:o,filename`'#13#10+
	'  -f, --force           Don''t ask for confirmation on drop table. Continue'#13#10+
	'                        even if we get an error'#13#10+
	'  -?, --help            Display this help and exit'#13#10+
	'  -h, --host=#          Connect to host'#13#10+
	'  -p, --password[=...]  Password to use when connecting to server'#13#10+
	'                        If password is not given it''s asked from the tty.'#13#10+
	'  -P  --port=...        Port number to use for connection'#13#10+
	'  -i, --sleep=sec       Execute commands again and again with a sleep between'#13#10+
	'  -s, --silent          Silently exit if one can''t connect to server'#13#10+
	'  -S  --socket=...      Socket file to use for connection'#13#10));
  write('  -u, --user=#          User for login if not current user'#13#10);
  write('  -V, --version         Output version information and exit'#13#10);

  writeln(PChar(#13#10'Where command is a one or more of: (Commands may be shortened)'#13#10+
	'  create databasename   Create a new database'#13#10+
	'  drop databasename     Delete a database and all its tables'#13#10+
	'  flush-hosts           Flush all cached hosts'#13#10+
	'  flush-logs            Flush all logs'#13#10+
	'  flush-tables          Flush all tables'#13#10+
	'  kill id,id,...        Kill mysql threads'#13#10+
	'  password new-password Change old password to new-password'#13#10+
	'  processlist           Show list of active threads in server'#13#10+
	'  reload                Reload grant tables'#13#10+
	'  refresh               Flush all tables and close and open logfiles'#13#10+
	'  shutdown              Take server down'#13#10+
	'  status                Gives a short status message from the server'#13#10+
	'  variables             Prints variables available'#13#10+
	'  version               Get version info from server'#13#10));
end;


function create_db(mysql:PMYSQL;db:PChar):Integer;
begin
  if mysql_create_db(mysql,db)<>0 then begin
    {my_printf_error(0,"create of '%s' failed\nerror: '%s'",MYF(ME_BELL),
		    db,mysql_error(mysql));}
    write(Format('create of ''%s'' failed'#7#13#10'error: ''%s'''#13#10,[db,mysql_error(mysql)]));
    Result := 1;
    exit;
  end;
  write(Format('Database "%s" created.'#13#10,[db]);
  Result := 0;
end;


function drop_db(mysql:PMYSQL;db:PChar):Integer;
var
  ch			: Char;
begin
  if not option_quick then begin
    writeln('Dropping the database is potentially a very bad thing to do.');
    writeln('Any data stored in the database will be destroyed.'#13#10);
    write(Format('Do you really want to drop the ''%s'' database [y/N]',[db]);
    while keypressed do Readkey; ch := UpCase(Readkey);
    if ch<>'Y' then begin
      writeln(#13#10'OK, aborting database drop!');
      Result := -1;
      exit;
    end;
  end;
  if mysql_drop_db(mysql,db)<>0 then begin
    {my_printf_error(0,"drop of '%s' failed;\nerror: '%s'",MYF(ME_BELL),
		    db,mysql_error(mysql));}
    writeln(Format('drop of ''%s'' failed;'#7#13#10'error: ''%s''',[db,mysql_error(mysql)]));
    Result := 1;
    exit;
  end;
  write(Format('Database "%s" dropped'#13#10,[db]));
  Result := 0;
end;


procedure nice_time(sec:Cardinal;buff:PChar);
var
  tmp			: Cardinal;
begin
  if sec >= 3600*24 then begin
    tmp := sec div (3600*24);
    dec(sec,3600*24*tmp);
    buff := int2str(tmp,buff,10);
    if tmp>1 then
      buff := StrECopy(buff,' days ')
    else
      buff := StrECopy(buff,' day ');
  end;
  if sec >= 3600 then begin
    tmp := sec div 3600;
    dec(sec,3600*tmp);
    buff := int2str(tmp,buff,10);
    if tmp>1 then
      buff := StrECopy(buff,' hours ')
    else
      buff := StrECopy(buff,' hour ');
  end;
  if sec >= 60 then begin
    tmp := sec div 60;
    dec(sec,60*tmp);
    buff := int2str(tmp,buff,10);
    buff := StrECopy(buff," min ");
  end;
  StrCopy(int2str(sec,buff,10),' sec');
end;

procedure print_header(result:PMYSQL_RES)
var
  field 		: PMYSQL_FIELD;
begin
  print_top(result);
  mysql_field_seek(result,0);
  write('|');
  field := mysql_fetch_field(result);
  while field<>nil do begin
    write(' ',field^.name,'|':(1+field^.max_length-StrLen(field^.name));
    field := mysql_fetch_field(result);
  end;
  writeln;
  print_top(result);
end;


procedure print_top(result:PMYSQL_RES)
var
  i,length		: Word;
  field 		: PMYSQL_FIELD;
begin
  write('+');
  mysql_field_seek(result,0);
  field := mysql_fetch_field(result);
  while field<>nil do begin
    length := StrLen(field^.name);
    if length>field^.max_length then
      field^.max_length := length;
    else
      length:=field^.max_length;
    for i := length+2 downto 1 do write('-');
    write('+');
    field := mysql_fetch_field(result);
  end;
  writeln;
end;


procedure print_row(result:PMYSQL_RES;cur:MYSQL_ROW);
var
  i, length,l		: Word;
  field 		: PMYSQL_FIELD;
begin
  write('|');
  mysql_field_seek(result,0);
  for i := 0 to mysql_num_fields(result)-1 do begin
    field := mysql_fetch_field(result);
    length := field^.max_length;
    if cur^[i]<>nil then
      Write(' ',cur^[i],'|':(1+length-l))
    else
      Write(' ','|':(1+length));
  end;
  writeln;
end;


